@@ -17,13 +17,12 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 
-import re
-import sys
+from __future__ import print_function
 from operator import itemgetter
 from PyQt4 import QtCore
 from picard import (PICARD_APP_NAME, PICARD_ORG_NAME, PICARD_VERSION,
                     version_to_string, version_from_string)
-from picard.util import LockableObject, rot13
+from picard.util import LockableObject
 
 
 class ConfigUpgradeError(Exception):
@@ -84,7 +83,7 @@ class ConfigSection(LockableObject):
         except:
             return default
         finally:
-            self.unlock()        
+            self.unlock()
 
 
 class Config(QtCore.QSettings):
@@ -92,8 +91,20 @@ class Config(QtCore.QSettings):
     """Configuration."""
 
     def __init__(self):
-        """Initializes the configuration."""
-        QtCore.QSettings.__init__(self, PICARD_ORG_NAME, PICARD_APP_NAME)
+        pass
+
+    def __initialize(self):
+        """Common initializer method for :meth:`from_app` and
+        :meth:`from_file`."""
+
+        # If there are no settings, copy existing settings from old format
+        # (registry on windows systems)
+        if not self.allKeys():
+            oldFormat = QtCore.QSettings(PICARD_ORG_NAME, PICARD_APP_NAME)
+            for k in oldFormat.allKeys():
+                self.setValue(k, oldFormat.value(k))
+            self.sync()
+
         self.application = ConfigSection(self, "application")
         self.setting = ConfigSection(self, "setting")
         self.persist = ConfigSection(self, "persist")
@@ -104,6 +115,27 @@ class Config(QtCore.QSettings):
         self._version = version_from_string(self.application["version"])
         self._upgrade_hooks = dict()
 
+    @classmethod
+    def from_app(cls, parent):
+        """Build a Config object using the default configuration file
+        location."""
+        this = cls()
+        QtCore.QSettings.__init__(this, QtCore.QSettings.IniFormat,
+                                  QtCore.QSettings.UserScope, PICARD_ORG_NAME,
+                                  PICARD_APP_NAME, parent)
+        this.__initialize()
+        return this
+
+    @classmethod
+    def from_file(cls, parent, filename):
+        """Build a Config object using a user-provided configuration file
+        path."""
+        this = cls()
+        QtCore.QSettings.__init__(this, filename, QtCore.QSettings.IniFormat,
+                                  parent)
+        this.__initialize()
+        return this
+
     def switchProfile(self, profilename):
         """Sets the current profile."""
         key = u"profile/%s" % (profilename,)
@@ -191,13 +223,6 @@ class Option(QtCore.QObject):
         return cls.registry.get((section, name))
 
 
-class PasswordOption(Option):
-
-    """Super l33t h3ckery!"""
-
-    convert = staticmethod(rot13)
-
-
 class TextOption(Option):
 
     convert = unicode
@@ -237,21 +262,16 @@ class IntListOption(Option):
         return map(int, value)
 
 
-_config = Config()
+config = None
+setting = None
+persist = None
 
-setting = _config.setting
-persist = _config.persist
 
-# http://pyqt.sourceforge.net/Docs/PyQt4/qsettings.html#fileName
-# QString QSettings.fileName (self)
-#
-# Returns the path where settings written using this QSettings object are stored.
-#
-# On Windows, if the format is QSettings.NativeFormat, the return value is a system registry path, not a file path.
-FILE_PATH = 0
-REGISTRY_PATH = 1
-storage = _config.fileName()
-if _config.format() == QtCore.QSettings.NativeFormat and sys.platform == "win32":
-    storage_type = REGISTRY_PATH
-else:
-    storage_type = FILE_PATH
+def _setup(app, filename=None):
+    global config, setting, persist
+    if filename is None:
+        config = Config.from_app(app)
+    else:
+        config = Config.from_file(app, filename)
+    setting = config.setting
+    persist = config.persist
