@@ -71,11 +71,17 @@ def _parse_attributes(attrs, reltype):
 
 
 def _relations_to_metadata(relation_lists, m):
+    use_credited_as = not config.setting["standardize_artists"]
     for relation_list in relation_lists:
         if relation_list.target_type == 'artist':
             for relation in relation_list.relation:
                 artist = relation.artist[0]
                 value, valuesort = _translate_artist_node(artist)
+                has_translation = (value != artist.name[0].text)
+                if not has_translation and use_credited_as and 'target_credit' in relation.children:
+                    credited_as = relation.target_credit[0].text
+                    if credited_as:
+                        value, valuesort = credited_as, credited_as
                 reltype = relation.type
                 attribs = []
                 if 'attribute_list' in relation.children:
@@ -161,13 +167,14 @@ def artist_credit_from_node(node):
     artistsort = ""
     artists = []
     artistssort = []
-    standardize_artists = config.setting["standardize_artists"]
+    use_credited_as = not config.setting["standardize_artists"]
     for credit in node.name_credit:
         a = credit.artist[0]
         translated, translated_sort = _translate_artist_node(a)
-        if translated != a.name[0].text:
+        has_translation = (translated != a.name[0].text)
+        if has_translation:
             name = translated
-        elif 'name' in credit.children and not standardize_artists:
+        elif use_credited_as and 'name' in credit.children:
             name = credit.name[0].text
         else:
             name = a.name[0].text
@@ -195,15 +202,31 @@ def artist_credit_to_metadata(node, m, release=False):
         m["artist"] = artist
         m["artistsort"] = artistsort
         m["artists"] = artists
-        m["~artists_sort"] = artistsort
+        m["~artists_sort"] = artistssort
+
+
+def country_list_from_node(node):
+    """Extract list of country codes from `release_event_list` node.
+    This is in contrast with `country` element, which has single release
+    event information.
+    """
+    if "release_event_list" in node.children:
+        country = []
+        for release_event in node.release_event_list[0].release_event:
+            try:
+                country.append(
+                    release_event.area[0].iso_3166_1_code_list[0].iso_3166_1_code[0].text)
+            except AttributeError:
+                pass
+        return country
 
 
 def label_info_from_node(node):
     labels = []
     catalog_numbers = []
-    if node.count != "0":
+    if node.children:
         for label_info in node.label_info:
-            if 'label' in label_info.children:
+            if 'label' in label_info.children and label_info.label[0].children:
                 label = label_info.label[0].name[0].text
                 if label not in labels:
                     labels.append(label)
@@ -239,7 +262,7 @@ def media_formats_from_node(node):
 
 def track_to_metadata(node, track):
     m = track.metadata
-    recording_to_metadata(node.recording[0], track)
+    recording_to_metadata(node.recording[0], m, track)
     m.add_unique('musicbrainz_trackid', node.id)
     # overwrite with data we have on the track
     for name, nodes in node.children.iteritems():
@@ -258,8 +281,7 @@ def track_to_metadata(node, track):
     m['~length'] = format_time(m.length)
 
 
-def recording_to_metadata(node, track):
-    m = track.metadata
+def recording_to_metadata(node, m, track=None):
     m.length = 0
     m.add_unique('musicbrainz_recordingid', node.id)
     for name, nodes in node.children.iteritems():
@@ -274,6 +296,17 @@ def recording_to_metadata(node, track):
             m['~recordingcomment'] = nodes[0].text
         elif name == 'artist_credit':
             artist_credit_to_metadata(nodes[0], m)
+            if 'name_credit' in nodes[0].children and track:
+                for name_credit in nodes[0].name_credit:
+                    if 'artist' in name_credit.children:
+                        for artist in name_credit.artist:
+                            trackartist = track.append_track_artist(artist.id)
+                            if 'tag_list' in artist.children:
+                                add_folksonomy_tags(artist.tag_list[0],
+                                                    trackartist)
+                            if 'user_tag_list' in artist.children:
+                                add_user_folksonomy_tags(artist.user_tag_list[0],
+                                                         trackartist)
         elif name == 'relation_list':
             _relations_to_metadata(nodes, m)
         elif name == 'tag_list':
@@ -284,6 +317,8 @@ def recording_to_metadata(node, track):
             add_isrcs_to_metadata(nodes[0], m)
         elif name == 'user_rating':
             m['~rating'] = nodes[0].text
+        elif name == 'video' and nodes[0].text == 'true':
+            m['~video'] = '1'
     m['~length'] = format_time(m.length)
 
 
@@ -299,7 +334,7 @@ def work_to_metadata(work, m):
     if 'language' in work.children:
         m.add_unique("language", work.language[0].text)
     if 'title' in work.children:
-        m.add("work", work.title[0].text)
+        m.add_unique("work", work.title[0].text)
     if 'relation_list' in work.children:
         _relations_to_metadata(work.relation_list, m)
 
@@ -318,6 +353,36 @@ def medium_to_metadata(node, m):
             m['media'] = nodes[0].text
 
 
+def artist_to_metadata(node, m):
+    """Make meatadata dict from a XML 'artist' node."""
+    m.add_unique("musicbrainz_artistid", node.id)
+    for name, nodes in node.children.iteritems():
+        if not nodes:
+            continue
+        if name == "name":
+            m["name"] = nodes[0].text
+        elif name == "area":
+            m["area"] = nodes[0].name[0].text
+        elif name == "gender":
+            m["gender"] = nodes[0].text
+        elif name == "life_span":
+            if "begin" in nodes[0].children:
+                m["begindate"] = nodes[0].begin[0].text
+            if "ended" in nodes[0].children:
+                ended = nodes[0].ended[0].text
+                if ended == "true" and "end" in nodes[0].children:
+                    m["enddate"] = nodes[0].end[0].text
+        elif name == "begin_area":
+            m["beginarea"] = nodes[0].name[0].text
+        elif name == "end_area":
+            m["endarea"] = nodes[0].name[0].text
+
+    try:
+        m["type"] = node.type
+    except AttributeError:
+        pass
+
+
 def release_to_metadata(node, m, album=None):
     """Make metadata dict from a XML 'release' node."""
     m.add_unique('musicbrainz_albumid', node.id)
@@ -334,6 +399,19 @@ def release_to_metadata(node, m, album=None):
             m['asin'] = nodes[0].text
         elif name == 'artist_credit':
             artist_credit_to_metadata(nodes[0], m, release=True)
+            # set tags from artists
+            if album is not None:
+                if 'name_credit' in nodes[0].children:
+                    for name_credit in nodes[0].name_credit:
+                        if 'artist' in name_credit.children:
+                            for artist in name_credit.artist:
+                                albumartist = album.append_album_artist(artist.id)
+                                if 'tag_list' in artist.children:
+                                    add_folksonomy_tags(artist.tag_list[0],
+                                                        albumartist)
+                                if 'user_tag_list' in artist.children:
+                                    add_user_folksonomy_tags(artist.user_tag_list[0],
+                                                             albumartist)
         elif name == 'date':
             m['date'] = nodes[0].text
         elif name == 'country':
@@ -342,7 +420,7 @@ def release_to_metadata(node, m, album=None):
             m['barcode'] = nodes[0].text
         elif name == 'relation_list':
             _relations_to_metadata(nodes, m)
-        elif name == 'label_info_list' and nodes[0].count != '0':
+        elif name == 'label_info_list' and nodes[0].children:
             m['label'], m['catalognumber'] = label_info_from_node(nodes[0])
         elif name == 'text_representation':
             if 'language' in nodes[0].children:
