@@ -24,6 +24,7 @@ from picard.similarity import similarity2
 from picard.util import (
     linear_combination_of_weights,
 )
+from picard.util.tags import PRESERVED_TAGS
 from picard.mbxml import artist_credit_from_node
 from picard.util.imagelist import ImageList
 
@@ -311,8 +312,8 @@ class Metadata(dict):
 
     def apply_func(self, func):
         for key, values in self.rawitems():
-            if not key.startswith("~"):
-                self[key] = map(func, values)
+            if key not in PRESERVED_TAGS:
+                self[key] = [func(value) for value in values]
 
     def strip_whitespace(self):
         """Strip leading/trailing whitespace.
