@@ -22,7 +22,9 @@ import os.path
 import sys
 from functools import partial
 from PyQt4 import QtGui
+from PyQt4.QtGui import QPalette, QColor
 from picard import config
+from picard.const import PICARD_URLS
 from picard.file import File
 from picard.script import ScriptParser, SyntaxError, ScriptError, UnknownFunction
 from picard.ui.options import OptionsPage, OptionsCheckError, register_options_page
@@ -32,7 +34,7 @@ from picard.ui.options.scripting import TaggerScriptSyntaxHighlighter
 
 
 _DEFAULT_FILE_NAMING_FORMAT = "$if2(%albumartist%,%artist%)/" \
-    "$if($ne(%albumartist%,),%album%/)" \
+    "$if($ne(%albumartist%,),%album%/,)" \
     "$if($gt(%totaldiscs%,1),%discnumber%-,)" \
     "$if($ne(%albumartist%,),$num(%tracknumber%,2) ,)" \
     "$if(%_multiartist%,%artist% - ,)" \
@@ -73,60 +75,16 @@ class RenamingOptionsPage(OptionsPage):
         self.ui.move_files.clicked.connect(self.update_examples)
         self.ui.move_files_to.editingFinished.connect(self.update_examples)
 
-        if not sys.platform == "win32":
-            self.ui.rename_files.stateChanged.connect(
-                partial(
-                    enabledSlot,
-                    self.ui.windows_compatibility.setEnabled
-                )
-            )
-
-        self.ui.move_files.stateChanged.connect(
+        self.ui.move_files.toggled.connect(
             partial(
                 enabledSlot,
-                self.ui.delete_empty_dirs.setEnabled
+                self.toggle_file_moving
             )
         )
-        self.ui.move_files.stateChanged.connect(
+        self.ui.rename_files.toggled.connect(
             partial(
                 enabledSlot,
-                self.ui.move_files_to.setEnabled
-            )
-        )
-        self.ui.move_files.stateChanged.connect(
-            partial(
-                enabledSlot,
-                self.ui.move_files_to_browse.setEnabled
-            )
-        )
-        self.ui.move_files.stateChanged.connect(
-            partial(
-                enabledSlot,
-                self.ui.move_additional_files.setEnabled
-            )
-        )
-        self.ui.move_files.stateChanged.connect(
-            partial(
-                enabledSlot,
-                self.ui.move_additional_files_pattern.setEnabled
-            )
-        )
-        self.ui.rename_files.stateChanged.connect(
-            partial(
-                enabledSlot,
-                self.ui.ascii_filenames.setEnabled
-            )
-        )
-        self.ui.rename_files.stateChanged.connect(
-            partial(
-                enabledSlot,
-                self.ui.file_naming_format.setEnabled
-            )
-        )
-        self.ui.rename_files.stateChanged.connect(
-            partial(
-                enabledSlot,
-                self.ui.file_naming_format_default.setEnabled
+                self.toggle_file_renaming
             )
         )
         self.ui.file_naming_format.textChanged.connect(self.check_formats)
@@ -134,6 +92,34 @@ class RenamingOptionsPage(OptionsPage):
         self.highlighter = TaggerScriptSyntaxHighlighter(self.ui.file_naming_format.document())
         self.ui.move_files_to_browse.clicked.connect(self.move_files_to_browse)
 
+        textEdit = self.ui.file_naming_format
+        self.textEditPaletteNormal = textEdit.palette()
+        self.textEditPaletteReadOnly = QPalette(self.textEditPaletteNormal)
+        disabled_color = self.textEditPaletteNormal.color(QPalette.Inactive, QPalette.Window)
+        self.textEditPaletteReadOnly.setColor(QPalette.Disabled, QPalette.Base, disabled_color)
+
+    def toggle_file_moving(self, state):
+        self.ui.delete_empty_dirs.setEnabled(state)
+        self.ui.move_files_to.setEnabled(state)
+        self.ui.move_files_to_browse.setEnabled(state)
+        self.ui.move_additional_files.setEnabled(state)
+        self.ui.move_additional_files_pattern.setEnabled(state)
+
+    def toggle_file_renaming(self, state):
+
+        self.ui.file_naming_format.setEnabled(state)
+        self.ui.file_naming_format_default.setEnabled(state)
+        self.ui.ascii_filenames.setEnabled(state)
+        self.ui.file_naming_format_group.setEnabled(state)
+        if not sys.platform == "win32":
+            self.ui.windows_compatibility.setEnabled(state)
+
+        if self.ui.file_naming_format.isEnabled():
+            self.ui.file_naming_format.setPalette(self.textEditPaletteNormal)
+        else:
+            self.ui.file_naming_format.setPalette(self.textEditPaletteReadOnly)
+
+
     def check_formats(self):
         self.test()
         self.update_examples()
@@ -149,10 +135,11 @@ class RenamingOptionsPage(OptionsPage):
             'move_files_to': os.path.normpath(unicode(self.ui.move_files_to.text()))
         }
         try:
-            if config.setting["enable_tagger_script"]:
-                script = config.setting["tagger_script"]
-                parser = ScriptParser()
-                parser.eval(script, file.metadata)
+            if config.setting["enable_tagger_scripts"]:
+                for s_pos, s_name, s_enabled, s_text in config.setting["list_of_scripts"]:
+                    if s_enabled and s_text:
+                        parser = ScriptParser()
+                        parser.eval(s_text, file.metadata)
             filename = file._make_filename(file.filename, file.metadata, settings)
             if not settings["move_files"]:
                 return os.path.basename(filename)
@@ -163,8 +150,6 @@ class RenamingOptionsPage(OptionsPage):
             return ""
         except TypeError:
             return ""
-        except UnknownFunction:
-            return ""
 
     def update_examples(self):
         # TODO: Here should be more examples etc.
@@ -184,6 +169,12 @@ class RenamingOptionsPage(OptionsPage):
         self.ui.move_files.setChecked(config.setting["move_files"])
         self.ui.ascii_filenames.setChecked(config.setting["ascii_filenames"])
         self.ui.file_naming_format.setPlainText(config.setting["file_naming_format"])
+        args = {
+            "picard-doc-scripting-url": PICARD_URLS['doc_scripting'],
+        }
+        text = _(u'<a href="%(picard-doc-scripting-url)s">Open Scripting'
+                 ' Documentation in your browser</a>') % args
+        self.ui.file_naming_format_documentation.setText(text)
         self.ui.move_files_to.setText(config.setting["move_files_to"])
         self.ui.move_files_to.setCursorPosition(0)
         self.ui.move_additional_files.setChecked(config.setting["move_additional_files"])
