@@ -2,9 +2,9 @@ brew tap cartr/qt4
 brew tap-pin cartr/qt4
 brew install gettext
 brew link gettext --force
-brew install qt
-brew install pyqt
-brew install libdiscid 
+brew install cartr/qt4/qt@4
+brew install cartr/qt4/pyqt@4
+brew install libdiscid
 pip install mutagen
 pip install discid
 pip install py2app
