@@ -4,7 +4,6 @@ import picard.formats
 import unittest
 import shutil
 
-
 from PyQt4 import QtCore
 from picard import config, log
 from picard.coverart.image import CoverArtImage, TagCoverArtImage
@@ -20,11 +19,12 @@ settings = {
     'id3v2_encoding': 'utf-8',
     'save_images_to_tags': True,
     'write_id3v23': False,
+    'id3v23_join_with': '/',
     'remove_ape_from_mp3': False,
     'remove_id3_from_flac': False,
     'rating_steps': 6,
     'rating_user_email': 'users@musicbrainz.org',
-    'save_only_front_images_to_tags': False,
+    'embed_only_one_front_image': False,
 }
 
 
@@ -88,10 +88,66 @@ class FormatsTest(unittest.TestCase):
             metadata[key] = value
         loaded_metadata = save_and_load_metadata(self.filename, metadata)
         for (key, value) in self.tags.iteritems():
-            #if key == 'comment:foo':
+            # if key == 'comment:foo':
             #    print "%r" % loaded_metadata
             self.assertEqual(loaded_metadata[key], value, '%s: %r != %r' % (key, loaded_metadata[key], value))
 
+    def test_delete_simple_tags(self):
+        if not self.original:
+            return
+        metadata = Metadata()
+        for (key, value) in self.tags.iteritems():
+            metadata[key] = value
+        if self.supports_ratings:
+            metadata['~rating'] = 1
+        original_metadata = save_and_load_metadata(self.filename, metadata)
+        metadata.delete('albumartist')
+        if self.supports_ratings:
+            metadata.delete('~rating')
+        new_metadata = save_and_load_metadata(self.filename, metadata)
+        self.assertIn('albumartist', original_metadata.keys())
+        self.assertNotIn('albumartist', new_metadata.keys())
+        if self.supports_ratings:
+            self.assertIn('~rating', original_metadata.keys())
+            self.assertNotIn('~rating', new_metadata.keys())
+
+    def test_delete_complex_tags(self):
+        if not self.original:
+            return
+        metadata = Metadata()
+
+        for (key, value) in self.tags.iteritems():
+            metadata[key] = value
+
+        original_metadata = save_and_load_metadata(self.filename, metadata)
+        metadata.delete('totaldiscs')
+        new_metadata = save_and_load_metadata(self.filename, metadata)
+
+        self.assertIn('totaldiscs', original_metadata)
+        if self.original.split(".")[1] == 'm4a':
+            self.assertEqual(u'0', new_metadata['totaldiscs'])
+        else:
+            self.assertNotIn('totaldiscs', new_metadata)
+
+    def test_delete_performer(self):
+        if not self.original:
+            return
+        if 'performer:guest vocal' in self.tags:
+            metadata = Metadata()
+            for (key, value) in self.tags.iteritems():
+                metadata[key] = value
+
+            metadata['performer:piano'] = 'Foo'
+
+            original_metadata = save_and_load_metadata(self.filename, metadata)
+            metadata.delete('performer:piano')
+            new_metadata = save_and_load_metadata(self.filename, metadata)
+
+            self.assertIn('performer:guest vocal', original_metadata)
+            self.assertIn('performer:guest vocal', new_metadata)
+            self.assertIn('performer:piano', original_metadata)
+            self.assertNotIn('performer:piano', new_metadata)
+
     def test_ratings(self):
         if not self.original or not self.supports_ratings:
             return
@@ -102,6 +158,24 @@ class FormatsTest(unittest.TestCase):
             loaded_metadata = save_and_load_metadata(self.filename, metadata)
             self.assertEqual(int(loaded_metadata['~rating']), rating, '~rating: %r != %r' % (loaded_metadata['~rating'], rating))
 
+    def test_guess_format(self):
+        if self.original:
+            fd, temp_file = mkstemp()
+            os.close(fd)
+            self.addCleanup(os.unlink, temp_file)
+            shutil.copy(self.original, temp_file)
+            audio = picard.formats.guess_format(temp_file)
+            audio_original = picard.formats.open(self.filename)
+            self.assertEqual(type(audio), type(audio_original))
+
+    def test_split_ext(self):
+        if self.original:
+            f = picard.formats.open(self.filename)
+            self.assertEqual(f._fixed_splitext(f.filename), os.path.splitext(f.filename))
+            self.assertEqual(f._fixed_splitext(f.EXTENSIONS[0]), ('', f.EXTENSIONS[0]))
+            self.assertEqual(f._fixed_splitext('.test'), os.path.splitext('.test'))
+            self.assertNotEqual(f._fixed_splitext(f.EXTENSIONS[0]), os.path.splitext(f.EXTENSIONS[0]))
+
 
 class FLACTest(FormatsTest):
     original = os.path.join('test', 'data', 'test.flac')
@@ -137,6 +211,7 @@ class FLACTest(FormatsTest):
         'comment:foo': 'Foo',
         'genre': 'Foo',
         'bpm': '80',
+        'key': 'E#m',
         'mood': 'Foo',
         'isrc': 'Foo',
         'copyright': 'Foo',
@@ -180,7 +255,8 @@ class WMATest(FormatsTest):
         'artist': 'Foo',
         'albumartist': 'Foo',
         'date': '2004',
-        'originaldate': '1980',
+        'originaldate': '1980-08-30',
+        'originalyear': '1980',
         'artist': 'Foo',
         'composer': 'Foo',
         'lyricist': 'Foo',
@@ -188,32 +264,34 @@ class WMATest(FormatsTest):
         #'performer:guest vocal': 'Foo',
         'remixer': 'Foo',
         #'arranger': 'Foo',
-        #'engineer': 'Foo',
+        'engineer': 'Foo',
         'producer': 'Foo',
-        #'djmixer': 'Foo',
-        #'mixer': 'Foo',
+        'djmixer': 'Foo',
+        'mixer': 'Foo',
         'grouping': 'Foo',
         'subtitle': 'Foo',
         'discsubtitle': 'Foo',
         'tracknumber': '2',
         #'totaltracks': '10',
         'discnumber': '1',
-        #'totaldiscs': '2',
-        #'compilation': '1',
+        'totaldiscs': '2',
+        'compilation': '1',
         'comment:': 'Foo',
         # FIXME: comment:foo is unsupported in our WMA implementation
         #'comment:foo': 'Foo',
         'genre': 'Foo',
         'bpm': '80',
+        'key': 'E#m',
         'mood': 'Foo',
         'isrc': 'Foo',
         'copyright': 'Foo',
         'lyrics': 'Foo',
-        #'media': 'Foo',
+        'media': 'Foo',
         'label': 'Foo',
-        #'catalognumber': 'Foo',
-        #'barcode': 'Foo',
+        'catalognumber': 'Foo',
+        'barcode': 'Foo',
         'encodedby': 'Foo',
+        'encodersettings': 'Foo',
         'albumsort': 'Foo',
         'albumartistsort': 'Foo',
         'artistsort': 'Foo',
@@ -230,15 +308,125 @@ class WMATest(FormatsTest):
         #'musicip_fingerprint': 'Foo',
         'releasestatus': 'Foo',
         'releasetype': 'Foo',
-        #'asin': 'Foo',
+        'asin': 'Foo',
         #'gapless': '1',
         #'podcast': '1',
         #'podcasturl': 'Foo',
         #'show': 'Foo',
+        'website': 'http://example.com',
     }
 
 
-class MP3Test(FormatsTest):
+class ID3Test(FormatsTest):
+
+    def test_id3_freeform_delete(self):
+        if not self.original:
+            return
+        metadata = Metadata()
+        for (key, value) in self.tags.iteritems():
+            metadata[key] = value
+
+        metadata['Foo'] = 'Foo'
+        original_metadata = save_and_load_metadata(self.filename, metadata)
+        metadata.delete('Foo')
+        new_metadata = save_and_load_metadata(self.filename, metadata)
+
+        self.assertIn('Foo', original_metadata)
+        self.assertNotIn('Foo', new_metadata)
+
+    def test_id3_ufid_delete(self):
+        if not self.original:
+            return
+        metadata = Metadata()
+        for (key, value) in self.tags.iteritems():
+            metadata[key] = value
+        metadata['musicbrainz_recordingid'] = "Foo"
+        original_metadata = save_and_load_metadata(self.filename, metadata)
+        metadata.delete('musicbrainz_recordingid')
+        new_metadata = save_and_load_metadata(self.filename, metadata)
+
+        self.assertIn('musicbrainz_recordingid', original_metadata)
+        self.assertNotIn('musicbrainz_recordingid', new_metadata)
+
+    def test_id3_multiple_freeform_delete(self):
+        if not self.original:
+            return
+        metadata = Metadata()
+        for (key, value) in self.tags.iteritems():
+            metadata[key] = value
+
+        metadata['Foo'] = 'Foo'
+        metadata['Bar'] = 'Foo'
+        metadata['FooBar'] = 'Foo'
+        original_metadata = save_and_load_metadata(self.filename, metadata)
+        metadata.delete('Foo')
+        metadata.delete('Bar')
+        new_metadata = save_and_load_metadata(self.filename, metadata)
+
+        self.assertIn('Foo', original_metadata)
+        self.assertIn('Bar', original_metadata)
+        self.assertIn('FooBar', original_metadata)
+        self.assertNotIn('Foo', new_metadata)
+        self.assertNotIn('Bar', new_metadata)
+        self.assertIn('FooBar', new_metadata)
+
+    def test_performer_duplication(self):
+        if not self.original:
+            return
+
+        def reset_id3_ver(): config.setting['write_id3v23'] = False
+
+        self.addCleanup(reset_id3_ver)
+        config.setting['write_id3v23'] = True
+        metadata = Metadata()
+        tags = {
+            'album': 'Foo',
+            'title': 'Foo',
+            'artist': 'Foo',
+            'performer:piano': 'Foo'
+        }
+
+        for (key, value) in tags.iteritems():
+            metadata[key] = value
+
+        original_metadata = save_and_load_metadata(self.filename, metadata)
+        new_metadata = save_and_load_metadata(self.filename, original_metadata)
+
+        self.assertEqual(len(new_metadata['performer:piano']), len(original_metadata['performer:piano']))
+
+    def test_comment_delete(self):
+        if not self.original:
+            return
+        metadata = Metadata()
+        for (key, value) in self.tags.iteritems():
+            metadata[key] = value
+        metadata['comment:bar'] = 'Foo'
+        original_metadata = save_and_load_metadata(self.filename, metadata)
+        metadata.delete('comment:bar')
+        new_metadata = save_and_load_metadata(self.filename, metadata)
+
+        self.assertIn('comment:foo', original_metadata)
+        self.assertIn('comment:bar', original_metadata)
+        self.assertIn('comment:foo', new_metadata)
+        self.assertNotIn('comment:bar', new_metadata)
+
+    def test_id3v23_simple_tags(self):
+        if not self.original:
+            return
+        def reset_to_id3v24(): config.setting['write_id3v23'] = False
+        config.setting['write_id3v23'] = True
+        self.addCleanup(reset_to_id3v24)
+        metadata = Metadata()
+        for (key, value) in self.tags.iteritems():
+            metadata[key] = value
+        loaded_metadata = save_and_load_metadata(self.filename, metadata)
+        for (key, value) in self.tags.iteritems():
+            # if key == 'comment:foo':
+            #    print "%r" % loaded_metadata
+            self.assertEqual(loaded_metadata[key], value, '%s: %r != %r' % (key, loaded_metadata[key], value))
+
+
+class MP3Test(ID3Test):
     original = os.path.join('test', 'data', 'test.mp3')
     supports_ratings = True
     tags = {
@@ -272,6 +460,7 @@ class MP3Test(FormatsTest):
         'comment:foo': 'Foo',
         'genre': 'Foo',
         'bpm': '80',
+        'key': 'E#m',
         'mood': 'Foo',
         'isrc': 'Foo',
         'copyright': 'Foo',
@@ -305,6 +494,143 @@ class MP3Test(FormatsTest):
     }
 
 
+class TTATest(ID3Test):
+    original = os.path.join('test', 'data', 'test.tta')
+    supports_ratings = True
+    tags = {
+        'album': 'Foo Bar',
+        'album': '1',
+        'title': 'Foo',
+        'artist': 'Foo',
+        'albumartist': 'Foo',
+        'date': '2004',
+        'originaldate': '1980',
+        'artist': 'Foo',
+        'composer': 'Foo',
+        'lyricist': 'Foo',
+        'conductor': 'Foo',
+        'performer:guest vocal': 'Foo',
+        'remixer': 'Foo',
+        'arranger': 'Foo',
+        'engineer': 'Foo',
+        'producer': 'Foo',
+        'djmixer': 'Foo',
+        'mixer': 'Foo',
+        'grouping': 'Foo',
+        'subtitle': 'Foo',
+        'discsubtitle': 'Foo',
+        'tracknumber': '2',
+        'totaltracks': '10',
+        'discnumber': '1',
+        'totaldiscs': '2',
+        'compilation': '1',
+        'comment:': 'Foo',
+        'comment:foo': 'Foo',
+        'genre': 'Foo',
+        'bpm': '80',
+        'key': 'E#m',
+        'mood': 'Foo',
+        'isrc': 'Foo',
+        'copyright': 'Foo',
+        'lyrics': 'Foo',
+        'media': 'Foo',
+        'label': 'Foo',
+        'catalognumber': 'Foo',
+        'barcode': 'Foo',
+        'encodedby': 'Foo',
+        'albumsort': 'Foo',
+        'albumartistsort': 'Foo',
+        'artistsort': 'Foo',
+        'titlesort': 'Foo',
+        'composersort': 'Foo',
+        #'showsort': 'Foo',
+        'musicbrainz_trackid': 'Foo',
+        'musicbrainz_albumid': 'Foo',
+        'musicbrainz_artistid': 'Foo',
+        'musicbrainz_albumartistid': 'Foo',
+        'musicbrainz_trmid': 'Foo',
+        'musicbrainz_discid': 'Foo',
+        'musicip_puid': 'Foo',
+        'musicip_fingerprint': 'Foo',
+        'releasestatus': 'Foo',
+        'releasetype': 'Foo',
+        'asin': 'Foo',
+        #'gapless': '1',
+        #'podcast': '1',
+        #'podcasturl': 'Foo',
+        #'show': 'Foo',
+    }
+
+
+if picard.formats.AiffFile:
+    class AIFFTest(ID3Test):
+        original = os.path.join('test', 'data', 'test.aiff')
+        supports_ratings = False
+        tags = {
+            'album': 'Foo Bar',
+            'album': '1',
+            'title': 'Foo',
+            'artist': 'Foo',
+            'albumartist': 'Foo',
+            'date': '2004',
+            'originaldate': '1980',
+            'artist': 'Foo',
+            'composer': 'Foo',
+            'lyricist': 'Foo',
+            'conductor': 'Foo',
+            'performer:guest vocal': 'Foo',
+            'remixer': 'Foo',
+            'arranger': 'Foo',
+            'engineer': 'Foo',
+            'producer': 'Foo',
+            'djmixer': 'Foo',
+            'mixer': 'Foo',
+            'grouping': 'Foo',
+            'subtitle': 'Foo',
+            # 'discsubtitle': 'Foo',
+            'tracknumber': '2',
+            'totaltracks': '10',
+            'discnumber': '1',
+            'totaldiscs': '2',
+            'compilation': '1',
+            'comment:': 'Foo',
+            'comment:foo': 'Foo',
+            'genre': 'Foo',
+            'bpm': '80',
+            'key': 'E#m',
+            'mood': 'Foo',
+            'isrc': 'Foo',
+            'copyright': 'Foo',
+            'lyrics': 'Foo',
+            'media': 'Foo',
+            'label': 'Foo',
+            'catalognumber': 'Foo',
+            'barcode': 'Foo',
+            'encodedby': 'Foo',
+            # 'albumsort': 'Foo',
+            'albumartistsort': 'Foo',
+            # 'artistsort': 'Foo',
+            # 'titlesort': 'Foo',
+            'composersort': 'Foo',
+            #'showsort': 'Foo',
+            'musicbrainz_trackid': 'Foo',
+            'musicbrainz_albumid': 'Foo',
+            'musicbrainz_artistid': 'Foo',
+            'musicbrainz_albumartistid': 'Foo',
+            'musicbrainz_trmid': 'Foo',
+            'musicbrainz_discid': 'Foo',
+            'musicip_puid': 'Foo',
+            'musicip_fingerprint': 'Foo',
+            'releasestatus': 'Foo',
+            'releasetype': 'Foo',
+            'asin': 'Foo',
+            #'gapless': '1',
+            #'podcast': '1',
+            #'podcasturl': 'Foo',
+            #'show': 'Foo',
+        }
+
+
 class OggVorbisTest(FormatsTest):
     original = os.path.join('test', 'data', 'test.ogg')
     supports_ratings = True
@@ -339,6 +665,7 @@ class OggVorbisTest(FormatsTest):
         'comment:foo': 'Foo',
         'genre': 'Foo',
         'bpm': '80',
+        'key': 'E#m',
         'mood': 'Foo',
         'isrc': 'Foo',
         'copyright': 'Foo',
@@ -407,6 +734,7 @@ class MP4Test(FormatsTest):
         #'comment:foo': 'Foo',
         'genre': 'Foo',
         'bpm': '80',
+        'key': 'E#m',
         'mood': 'Foo',
         'isrc': 'Foo',
         'copyright': 'Foo',
@@ -474,6 +802,7 @@ class WavPackTest(FormatsTest):
         'comment:foo': 'Foo',
         'genre': 'Foo',
         'bpm': '80',
+        'key': 'E#m',
         'mood': 'Foo',
         'isrc': 'Foo',
         'copyright': 'Foo',
@@ -506,8 +835,145 @@ class WavPackTest(FormatsTest):
         #'show': 'Foo',
     }
 
+
+class MusepackSV7Test(FormatsTest):
+    original = os.path.join('test', 'data', 'test-sv7.mpc')
+    supports_ratings = False
+    tags = {
+        'album': 'Foo Bar',
+        'album': '1',
+        'title': 'Foo',
+        'artist': 'Foo',
+        'albumartist': 'Foo',
+        'date': '2004',
+        #'originaldate': '1980',
+        'artist': 'Foo',
+        'composer': 'Foo',
+        'lyricist': 'Foo',
+        'conductor': 'Foo',
+        'performer:guest vocal': 'Foo',
+        'remixer': 'Foo',
+        'arranger': 'Foo',
+        'engineer': 'Foo',
+        'producer': 'Foo',
+        'djmixer': 'Foo',
+        'mixer': 'Foo',
+        'grouping': 'Foo',
+        'subtitle': 'Foo',
+        'discsubtitle': 'Foo',
+        'tracknumber': '2',
+        'totaltracks': '10',
+        'discnumber': '1',
+        'totaldiscs': '2',
+        'compilation': '1',
+        'comment:': 'Foo',
+        'comment:foo': 'Foo',
+        'genre': 'Foo',
+        'bpm': '80',
+        'key': 'E#m',
+        'mood': 'Foo',
+        'isrc': 'Foo',
+        'copyright': 'Foo',
+        'lyrics': 'Foo',
+        'media': 'Foo',
+        'label': 'Foo',
+        'catalognumber': 'Foo',
+        'barcode': 'Foo',
+        'encodedby': 'Foo',
+        'albumsort': 'Foo',
+        'albumartistsort': 'Foo',
+        'artistsort': 'Foo',
+        'titlesort': 'Foo',
+        'composersort': 'Foo',
+        #'showsort': 'Foo',
+        'musicbrainz_trackid': 'Foo',
+        'musicbrainz_albumid': 'Foo',
+        'musicbrainz_artistid': 'Foo',
+        'musicbrainz_albumartistid': 'Foo',
+        'musicbrainz_trmid': 'Foo',
+        'musicbrainz_discid': 'Foo',
+        'musicip_puid': 'Foo',
+        #'musicip_fingerprint': 'Foo',
+        'releasestatus': 'Foo',
+        'releasetype': 'Foo',
+        'asin': 'Foo',
+        #'gapless': '1',
+        #'podcast': '1',
+        #'podcasturl': 'Foo',
+        #'show': 'Foo',
+    }
+
+
+class MusepackSV8Test(FormatsTest):
+    original = os.path.join('test', 'data', 'test-sv8.mpc')
+    supports_ratings = False
+    tags = {
+        'album': 'Foo Bar',
+        'album': '1',
+        'title': 'Foo',
+        'artist': 'Foo',
+        'albumartist': 'Foo',
+        'date': '2004',
+        #'originaldate': '1980',
+        'artist': 'Foo',
+        'composer': 'Foo',
+        'lyricist': 'Foo',
+        'conductor': 'Foo',
+        'performer:guest vocal': 'Foo',
+        'remixer': 'Foo',
+        'arranger': 'Foo',
+        'engineer': 'Foo',
+        'producer': 'Foo',
+        'djmixer': 'Foo',
+        'mixer': 'Foo',
+        'grouping': 'Foo',
+        'subtitle': 'Foo',
+        'discsubtitle': 'Foo',
+        'tracknumber': '2',
+        'totaltracks': '10',
+        'discnumber': '1',
+        'totaldiscs': '2',
+        'compilation': '1',
+        'comment:': 'Foo',
+        'comment:foo': 'Foo',
+        'genre': 'Foo',
+        'bpm': '80',
+        'key': 'E#m',
+        'mood': 'Foo',
+        'isrc': 'Foo',
+        'copyright': 'Foo',
+        'lyrics': 'Foo',
+        'media': 'Foo',
+        'label': 'Foo',
+        'catalognumber': 'Foo',
+        'barcode': 'Foo',
+        'encodedby': 'Foo',
+        'albumsort': 'Foo',
+        'albumartistsort': 'Foo',
+        'artistsort': 'Foo',
+        'titlesort': 'Foo',
+        'composersort': 'Foo',
+        #'showsort': 'Foo',
+        'musicbrainz_trackid': 'Foo',
+        'musicbrainz_albumid': 'Foo',
+        'musicbrainz_artistid': 'Foo',
+        'musicbrainz_albumartistid': 'Foo',
+        'musicbrainz_trmid': 'Foo',
+        'musicbrainz_discid': 'Foo',
+        'musicip_puid': 'Foo',
+        #'musicip_fingerprint': 'Foo',
+        'releasestatus': 'Foo',
+        'releasetype': 'Foo',
+        'asin': 'Foo',
+        #'gapless': '1',
+        #'podcast': '1',
+        #'podcasturl': 'Foo',
+        #'show': 'Foo',
+    }
+
+
 cover_settings = {
-    'save_only_front_images_to_tags': True,
+    'embed_only_one_front_image': True,
 }
 
 
@@ -777,7 +1243,7 @@ class TestCoverArt(unittest.TestCase):
             self._tear_down()
 
     def _test_cover_art_types_only_front(self, filename, expect):
-        self._set_up(filename, {'save_only_front_images_to_tags': True})
+        self._set_up(filename, {'embed_only_one_front_image': True})
         try:
             f = picard.formats.open(self.filename)
             f._save(self.filename, self._cover_metadata())
